/*
 * ainfo.cpp:
 *
 * print information about an aff file
 */

/*
 * Copyright (c) 2005
 *	Simson L. Garfinkel and Basis Technology, Inc. 
 *      All rights reserved.
 *
 * This code is derrived from software contributed by
 * Simson L. Garfinkel
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. [Omitted.]
 * 4. Neither the name of Simson Garfinkel, Basis Technology, or other
 *    contributors to this program may be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY SIMSON GARFINKEL, BASIS TECHNOLOGY,
 * AND CONTRIBUTORS ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SIMSON GARFINKEL, BAIS TECHNOLOGy,
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.  
 */


#include "afflib.h"

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <err.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <db.h>
#include <fcntl.h>
#include <limits.h>
#include <string.h>
#include <zlib.h>
#include <openssl/md5.h>
#include <openssl/sha.h>
#include <assert.h>

char *progname = "acat";

int opt_validate = 0;
int opt_info = 1;

void usage()
{
    printf("usage: %s [options] infile\n",progname);
    printf("         -v = validate the file's md5 and sha-1 codes\n");
    printf("         -i = print informaton about each segment\n");
    exit(0);
}



void validate(AFFILE *af)
{
    long long bytes_read=0;

    /* Set up the hash machinery */
    MD5_CTX md5;
    MD5_Init(&md5);

    SHA_CTX sha;
    SHA1_Init(&sha);
    
    unsigned long long total_bytes = 0;
    while(!af_eof(af)){
	unsigned char buf[65536];		// a decent size
	size_t bytes = af_read(af,buf,sizeof(buf));
	total_bytes += bytes;
	MD5_Update(&md5,buf,bytes);
	SHA1_Update(&sha,buf,bytes);
    }
    
    /* Finish the hash calculations and write to the db */
    unsigned char *md5_data;
    size_t md5len;
    unsigned char ibuf[16];
    char buf[256];
    unsigned long arg;
    
    MD5_Final(ibuf,&md5);
    printf("md5: %s ",af_hexbuf(buf,ibuf,16));
    if(af_get_segv(af,AF_MD5,&arg,&md5_data,&md5len)==0){
	if(md5len==16 && !memcmp((const char *)md5_data,(const char *)ibuf,16)){
	    printf(" MATCH\n");
	}
	else {
	    printf(" NO MATCH!\n");
	}
	free(md5_data);
    }
    else {
	printf("(no MD5 in AFF file)\n");
    }
    
    
    unsigned char *sha1_data;
    size_t sha1len;

    SHA1_Final(ibuf,&sha);
    printf("sha1: %s ",af_hexbuf(buf,ibuf,20));
    if(af_get_segv(af,AF_MD5,&arg,&sha1_data,&sha1len)==0){
	if(md5len==20 && !memcmp((const char *)sha1_data,(const char *)ibuf,20)){
	    printf(" MATCH\n");
	}
	else {
	    printf(" NO MATCH!\n");
	}
	free(sha1_data);
    }
    else {
	printf("(no SHA1 in AFF file)\n");
    }
    
    struct stat sb;
    if(fstat(fileno(af->aseg),&sb)){
	err(1,"fstat");
    }
    
    printf("AFF file is %qd bytes. Actual compression radio with overhead: %4.1f%%\n",
	   sb.st_size,((float)bytes_read-sb.st_size)*100.0/bytes_read);
    af_close(af);
}

int get_info(AFFILE *af)
{
    printf("Segment            arg     length    data\n");
    printf("=======       ========     ======    =====\n");

    char *segname;
    unsigned long arg;
    unsigned char *data;
    unsigned long datalen;
    unsigned long long q;
    int segs = 0;
    while(af_get_next_segv(af,&segname,&arg,0,&datalen)==0){
	segs++;
	printf("%-10s    %8ld   %8ld    ",segname,arg,datalen);
	if(datalen==8){		// might as well print it as an 8-byte
	    af_backspace(af);
	    if(af_get_segq(af,segname,&q)){
		printf(">> can't read quad word '%s' <<",segname);
	    }
	    else{
		printf("%qd",q);
	    }
	}
	if(datalen==16 || datalen==20){
	    af_backspace(af);
	    if(af_get_segv(af,segname,&arg,&data,&datalen)){
		printf(">> can't read data '%s' <<",segname);
	    }
	    else {
		char buf[21];
		printf("%s",af_hexbuf(buf,data,datalen));
		free(data);
	    }
	}
	free(segname);
	printf("\n");
    }
    return 0;
}
	 



int main(int argc,char **argv)
{
     int bflag, ch, fd;
     const char *infile;
     unsigned char vbuf[256];

     progname = argv[0];
     bflag = 0;
     while ((ch = getopt(argc, argv, "s:h?vi")) != -1) {
             switch (ch) {
	     case 'h':
             case '?':
             default:
		 usage();
		 break;
	     case 'i':
		 opt_info++;
		 break;
	     case 'v':
		 opt_validate++;
		 break;
             }
     }
     argc -= optind;
     argv += optind;

     if(argc<1){
	 usage();
     }

     infile = *argv++;
     argc--;

     AFFILE *af = af_open(infile,O_RDONLY,0);
     if(!af) err(1,infile);

     if(opt_info) get_info(af);
     if(opt_validate) validate(af);
}


